@@ -17,9 +17,16 @@ from PIL import __version__ as pil_version
 from pypdf import PdfMerger, PdfReader, PdfWriter
 from pypdf.constants import PageAttributes as PG
 from pypdf.errors import PdfReadError, PdfReadWarning
-from pypdf.generic import ContentStream, NameObject, read_object
+from pypdf.generic import (
+    ArrayObject,
+    ContentStream,
+    DictionaryObject,
+    NameObject,
+    TextStringObject,
+    read_object,
+)
 
-from . import get_data_from_url, normalize_warnings
+from . import PILContext, get_data_from_url, normalize_warnings
 
 TESTS_ROOT = Path(__file__).parent.resolve()
 PROJECT_ROOT = TESTS_ROOT.parent
@@ -60,7 +67,7 @@ def test_basic_features(tmp_path):
 
     # add some Javascript to launch the print window on opening this PDF.
     # the password dialog may prevent the print dialog from being shown,
-    # comment the the encription lines, if that's the case, to try this out
+    # comment the encryption lines, if that's the case, to try this out
     writer.add_js("this.print({bUI:true,bSilent:false,bShrinkToFit:true});")
 
     # encrypt your new PDF and add a password
@@ -540,7 +547,11 @@ def test_get_fields_warns(tmp_path, caplog, url, name):
         retrieved_fields = reader.get_fields(fileobj=fp)
 
     assert retrieved_fields == {}
-    assert normalize_warnings(caplog.text) == ["Object 2 0 not defined."]
+    assert normalize_warnings(caplog.text) == [
+        "Ignoring wrong pointing object 1 65536 (offset 0)",
+        "Ignoring wrong pointing object 2 65536 (offset 0)",
+        "Object 2 0 not defined.",
+    ]
 
 
 @pytest.mark.enable_socket()
@@ -661,12 +672,13 @@ def test_image_extraction(url, name):
     if not root.exists():
         root.mkdir()
 
-    for page in reader.pages:
-        for image in page.images:
-            filename = root / image.name
-            with open(filename, "wb") as img:
-                img.write(image.data)
-            images_extracted.append(filename)
+    with PILContext():
+        for page in reader.pages:
+            for image in page.images:
+                filename = root / image.name
+                with open(filename, "wb") as img:
+                    img.write(image.data)
+                images_extracted.append(filename)
 
     # Cleanup
     do_cleanup = True  # set this to False for manual inspection
@@ -1038,7 +1050,7 @@ Division / Dept: 50 / 170
 Season: SUMMER-B 2023"""
         in reader.pages[0].extract_text()
     )
-    # currently threre is still a white space on last line missing
+    # currently there is still a white space on last line missing
     # so we can not do a full comparison.
 
 
@@ -1108,3 +1120,169 @@ def test_text_extraction_invalid_mode():
     reader = PdfReader(pdf_path)
     with pytest.raises(ValueError, match="Invalid text extraction mode"):
         reader.pages[0].extract_text(extraction_mode="foo")  # type: ignore
+
+
+@pytest.mark.enable_socket()
+def test_get_page_showing_field():
+    """
+    Uses testfile from #2452 in order to get fields on multiple pages,
+        choices boxes,...
+    """
+    url = "https://github.com/py-pdf/pypdf/files/14031491/Form_Structure_v50.pdf"
+    name = "iss2452.pdf"
+    reader = PdfReader(BytesIO(get_data_from_url(url, name)))
+    writer = PdfWriter(clone_from=reader)
+
+    # validate with Field:  only works on Reader (no get_fields on writer yet)
+    fld = reader.get_fields()
+    assert [
+        p.page_number for p in reader.get_pages_showing_field(fld["FormVersion"])
+    ] == [0]
+
+    # validate with dictionary object
+    # NRCategory field is a radio box
+    assert [
+        p.page_number
+        for p in reader.get_pages_showing_field(
+            reader.trailer["/Root"]["/AcroForm"]["/Fields"][8].get_object()
+        )
+    ] == [0, 0, 0, 0, 0]
+    assert [
+        p.page_number
+        for p in writer.get_pages_showing_field(
+            writer._root_object["/AcroForm"]["/Fields"][8].get_object()
+        )
+    ] == [0, 0, 0, 0, 0]
+
+    # validate with IndirectObject
+    # SiteID field is a textbox on multiple pages
+    assert [
+        p.page_number
+        for p in reader.get_pages_showing_field(
+            reader.trailer["/Root"]["/AcroForm"]["/Fields"][99]
+        )
+    ] == [0, 1]
+    assert [
+        p.page_number
+        for p in writer.get_pages_showing_field(
+            writer._root_object["/AcroForm"]["/Fields"][99]
+        )
+    ] == [0, 1]
+    # test directly on the widget:
+    assert [
+        p.page_number
+        for p in reader.get_pages_showing_field(
+            reader.trailer["/Root"]["/AcroForm"]["/Fields"][99]["/Kids"][1]
+        )
+    ] == [1]
+    assert [
+        p.page_number
+        for p in writer.get_pages_showing_field(
+            writer._root_object["/AcroForm"]["/Fields"][99]["/Kids"][1]
+        )
+    ] == [1]
+
+    # Exceptions:
+    # Invalid Object
+    with pytest.raises(ValueError) as exc:
+        reader.get_pages_showing_field(None)
+    with pytest.raises(ValueError) as exc:
+        writer.get_pages_showing_field(None)
+    assert "field type is invalid" in exc.value.args[0]
+
+    # Damage Field
+    del reader.trailer["/Root"]["/AcroForm"]["/Fields"][1].get_object()["/FT"]
+    del writer._root_object["/AcroForm"]["/Fields"][1].get_object()["/FT"]
+    with pytest.raises(ValueError) as exc:
+        reader.get_pages_showing_field(
+            reader.trailer["/Root"]["/AcroForm"]["/Fields"][1]
+        )
+    with pytest.raises(ValueError) as exc:
+        writer.get_pages_showing_field(writer._root_object["/AcroForm"]["/Fields"][1])
+    assert "field is not valid" in exc.value.args[0]
+
+    # missing Parent in field
+    del reader.trailer["/Root"]["/AcroForm"]["/Fields"][99]["/Kids"][1].get_object()[
+        "/Parent"
+    ]
+    del writer._root_object["/AcroForm"]["/Fields"][99]["/Kids"][1].get_object()[
+        "/Parent"
+    ]
+    with pytest.raises(ValueError) as exc:
+        reader.get_pages_showing_field(
+            reader.trailer["/Root"]["/AcroForm"]["/Fields"][1]
+        )
+    with pytest.raises(ValueError) as exc:
+        writer.get_pages_showing_field(writer._root_object["/AcroForm"]["/Fields"][1])
+
+    # remove "/P" (optional)
+    del reader.trailer["/Root"]["/AcroForm"]["/Fields"][8]["/Kids"][1].get_object()[
+        "/P"
+    ]
+    del writer._root_object["/AcroForm"]["/Fields"][8]["/Kids"][1].get_object()["/P"]
+    assert [
+        p.page_number
+        for p in reader.get_pages_showing_field(
+            reader.trailer["/Root"]["/AcroForm"]["/Fields"][8]["/Kids"][1]
+        )
+    ] == [0]
+    assert [
+        p.page_number
+        for p in writer.get_pages_showing_field(
+            writer._root_object["/AcroForm"]["/Fields"][8]["/Kids"][1]
+        )
+    ] == [0]
+    assert [
+        p.page_number
+        for p in reader.get_pages_showing_field(
+            reader.trailer["/Root"]["/AcroForm"]["/Fields"][8].get_object()
+        )
+    ] == [0, 0, 0, 0, 0]
+    assert [
+        p.page_number
+        for p in writer.get_pages_showing_field(
+            writer._root_object["/AcroForm"]["/Fields"][8].get_object()
+        )
+    ] == [0, 0, 0, 0, 0]
+
+    # Grouping fields
+    reader.trailer["/Root"]["/AcroForm"]["/Fields"][-1].get_object()[
+        NameObject("/Kids")
+    ] = ArrayObject([reader.trailer["/Root"]["/AcroForm"]["/Fields"][0]])
+    del reader.trailer["/Root"]["/AcroForm"]["/Fields"][-1].get_object()["/T"]
+    del reader.trailer["/Root"]["/AcroForm"]["/Fields"][-1].get_object()["/P"]
+    del reader.trailer["/Root"]["/AcroForm"]["/Fields"][-1].get_object()["/Subtype"]
+    writer._root_object["/AcroForm"]["/Fields"].append(
+        writer._add_object(
+            DictionaryObject(
+                {
+                    NameObject("/T"): TextStringObject("grouping"),
+                    NameObject("/FT"): NameObject("/Tx"),
+                    NameObject("/Kids"): ArrayObject(
+                        [reader.trailer["/Root"]["/AcroForm"]["/Fields"][0]]
+                    ),
+                }
+            )
+        )
+    )
+    assert [
+        p.page_number
+        for p in reader.get_pages_showing_field(
+            reader.trailer["/Root"]["/AcroForm"]["/Fields"][-1]
+        )
+    ] == []
+    assert [
+        p.page_number
+        for p in writer.get_pages_showing_field(
+            writer._root_object["/AcroForm"]["/Fields"][-1]
+        )
+    ] == []
+
+
+@pytest.mark.enable_socket()
+def test_extract_empty_page():
+    """Cf #2533"""
+    url = "https://github.com/py-pdf/pypdf/files/14718318/test.pdf"
+    name = "iss2533.pdf"
+    reader = PdfReader(BytesIO(get_data_from_url(url, name)))
+    assert reader.pages[1].extract_text(extraction_mode="layout") == ""
